@@ -58,7 +58,7 @@ _LOGGER = logging.getLogger(__name__)
 TIMEOUT = timedelta(seconds=30)
 
 class Connection:
-    """ Connection to VW-Group Connect services """
+    """ Connection to Connect services """
   # Init connection class
     def __init__(self, session, username, password, fulldebug=False, interval=timedelta(minutes=5)):
         """ Initialize """
@@ -96,6 +96,7 @@ class Connection:
         _LOGGER.debug('Initiating new login')
         # Remove cookies and re-init headers as we are doing a new login
         self._clear_cookies()
+        self._vehicles.clear()
         self._session_headers = HEADERS_SESSION.copy()
         self._session_auth_headers = HEADERS_AUTH.copy()
 
@@ -290,19 +291,19 @@ class Connection:
                 data = mailform
             )
             if req.status != 200:
-                raise SkodaException('POST password request failed')
+                raise SkodaException('POST username request failed')
             try:
                 response_data = await req.text()
                 responseSoup = BeautifulSoup(response_data, 'html.parser')
                 for t in responseSoup.find('form', id='credentialsForm').find_all('input', type='hidden'):
                     _LOGGER.debug(f'Found item: {t["name"], t["value"]}')
                 pwform = dict([(t['name'],t['value']) for t in responseSoup.find('form', id='credentialsForm').find_all('input', type='hidden')])
-                #pwform = dict([(t['name'],t['value']) for t in responseSoup.find('form', id='credentialsForm').find_all('input', type='hidden')])
                 pwform['password'] = self._session_auth_password
                 pw_url = authissuer+responseSoup.find('form', id='credentialsForm').get('action')
             except Exception as e:
-                _LOGGER.error('Failed to extract password login form.')
-                raise SkodaException(e)
+                if responseSoup.find('form', id='credentialsForm') is None:
+                    raise SkodaAuthenticationException("Invalid username")
+                raise SkodaAuthenticationException("Invalid username or service unavailable")
 
             # POST password
             # https://identity.vwgroup.io/signin-service/v1/{CLIENT_ID}/login/authenticate
@@ -385,7 +386,7 @@ class Connection:
             )
             if req.status != 200:
                 raise SkodaException('Token exchange failed')
-            # Save tokens as "identity", theese are tokens representing the user
+            # Save tokens according to requested client
             self._session_tokens[client] = await req.json()
             if 'error' in self._session_tokens[client]:
                 error = self._session_tokens[client].get('error', '')
@@ -617,8 +618,11 @@ class Connection:
             # Get all Vehicle objects and update in parallell
             updatelist = []
             for vehicle in self.vehicles:
-                _LOGGER.debug(f'Adding {vehicle.vin} for data refresh')
-                updatelist.append(vehicle.update())
+                if vehicle.vin not in updatelist:
+                    _LOGGER.debug(f'Adding {vehicle.vin} for data refresh')
+                    updatelist.append(vehicle.update())
+                else:
+                    _LOGGER.debug(f'VIN {vehicle.vin} is already queued for data refresh')
             # Wait for all data updates to complete
             _LOGGER.debug('Calling update function for all vehicles')
             await asyncio.gather(*updatelist)
@@ -782,7 +786,7 @@ class Connection:
             _LOGGER.warning(f'Could not fetch position, error: {error}')
         return False
 
-    async def getTimers(self, vin):
+    async def getDeparturetimer(self, vin):
         """Get departure timers."""
         if not await self.validate_tokens:
             return False
@@ -793,7 +797,7 @@ class Connection:
                 vin = vin
             )
             if response.get('timer', {}):
-                data = {'timers': response.get('timer', {})}
+                data = {'departuretimer': response.get('timer', {})}
                 return data
             elif response.get('status_code', {}):
                 _LOGGER.warning(f'Could not fetch timers, HTTP status code: {response.get("status_code")}')
@@ -803,6 +807,24 @@ class Connection:
             _LOGGER.warning(f'Could not fetch timers, error: {error}')
         return False
 
+    async def getTimers(self, vin):
+        """Get timers data (New Skoda API)."""
+        if not await self.validate_tokens:
+            return False
+        try:
+            await self.set_token('connect')
+            response = await self.get('https://api.connect.skoda-auto.cz/api/v1/air-conditioning/$vin/timers', vin = vin)
+            if response.get('timers', []):
+                data = {'timers': response.get('timers', [])}
+                return data
+            elif chargerStatus.get('status_code', {}):
+                _LOGGER.warning(f'Could not fetch timers, HTTP status code: {response.get("status_code")}')
+            else:
+                _LOGGER.info('Unhandled error while trying to fetch timers data')
+        except Exception as error:
+            _LOGGER.warning(f'Could not fetch timers, error: {error}')
+        return False
+
     async def getClimater(self, vin):
         """Get climatisation data."""
         if not await self.validate_tokens:
@@ -824,6 +846,32 @@ class Connection:
             _LOGGER.warning(f'Could not fetch climatisation, error: {error}')
         return False
 
+    async def getAirConditioning(self, vin):
+        """Get air-conditioning data (Skoda native API)."""
+        if not await self.validate_tokens:
+            return False
+        try:
+            await self.set_token('connect')
+            airconStatus = self.get('https://api.connect.skoda-auto.cz/api/v1/air-conditioning/$vin/status', vin = vin)
+            airconSettings = self.get('https://api.connect.skoda-auto.cz/api/v1/air-conditioning/$vin/settings', vin = vin)
+            airconData = await asyncio.gather(airconStatus, airconSettings)
+
+            if airconData[0].get('state', {}):
+                data = {'airConditioning': airconData[0]}
+                if len(airconData) >= 2:
+                    data['airConditioningSettings'] = airconData[1]
+                if len(airconData) >= 3:
+                    data['airConditioningTimers'] = airconData[2]
+                _LOGGER.info(f"Returning with data {data}")
+                return data
+            elif response.get('status_code', {}):
+                _LOGGER.warning(f'Could not fetch air-conditioning, HTTP status code: {response.get("status_code")}')
+            else:
+                _LOGGER.info('Unhandled error while trying to fetch air-conditioning data')
+        except Exception as error:
+            _LOGGER.warning(f'Could not fetch air-conditioning, error: {error}')
+        return False
+
     async def getCharger(self, vin):
         """Get charger data."""
         if not await self.validate_tokens:
@@ -851,15 +899,17 @@ class Connection:
             return False
         try:
             await self.set_token('connect')
-            response = await self.get(
-                'https://api.connect.skoda-auto.cz/api/v1/charging/$vin/status',
-                vin = vin
-            )
-            if response.get('battery', {}):
-                _LOGGER.debug(f'Got vehicle data {response}')
+            chargerStatus = self.get('https://api.connect.skoda-auto.cz/api/v1/charging/$vin/status', vin = vin)
+            chargerSettings = self.get('https://api.connect.skoda-auto.cz/api/v1/charging/$vin/settings', vin = vin)
+            chargingData = await asyncio.gather(chargerStatus, chargerSettings)
+
+            if chargingData[0].get('battery', {}) or chargingData[1].get('maxChargeCurrentAc', {}):
+                _LOGGER.debug(f'Got vehicle charging data')
+                response = chargingData[0]
+                response['chargerSettings'] = chargingData[1]
                 return response
-            elif response.get('status_code', {}):
-                _LOGGER.warning(f'Could not fetch charging, HTTP status code: {response.get("status_code")}')
+            elif chargerStatus.get('status_code', {}):
+                _LOGGER.warning(f'Could not fetch charging, HTTP status code: {chargerStatus.get("status_code")}')
             else:
                 _LOGGER.info('Unhandled error while trying to fetch charging data')
         except Exception as error:
@@ -889,7 +939,7 @@ class Connection:
 
     async def get_request_status(self, vin, sectionId, requestId):
         """Return status of a request ID for a given section ID."""
-        if self.logged_in == False:
+        if not self.logged_in:
             if not await self.doLogin():
                 _LOGGER.warning(f'Login for {BRAND} account failed!')
                 raise SkodaLoginFailedException(f'Login for {BRAND} account failed')
@@ -899,8 +949,11 @@ class Connection:
                 if not await self.doLogin():
                     _LOGGER.warning(f'Login for {BRAND} account failed!')
                     raise SkodaLoginFailedException(f'Login for {BRAND} account failed')
-            await self.set_token('vwg')
-            if sectionId == 'climatisation':
+            # Requests for Skoda Native API
+            if sectionId in ['charging',  'air-conditioning']:
+                url = 'https://api.connect.skoda-auto.cz/api/v1/$sectionId/operation-requests/$requestId'
+            # Requests for VW-Group API
+            elif sectionId == 'climatisation':
                 url = f'fs-car/bs/$sectionId/v1/{BRAND}/{COUNTRY}/vehicles/$vin/climater/actions/$requestId'
             elif sectionId == 'batterycharge':
                 url = f'fs-car/bs/$sectionId/v1/{BRAND}/{COUNTRY}/vehicles/$vin/charger/actions/$requestId'
@@ -913,23 +966,34 @@ class Connection:
             url = re.sub('\$sectionId', sectionId, url)
             url = re.sub('\$requestId', requestId, url)
 
+            # Set token according to API origin
+            if sectionId in ['charging', 'air-conditioning']:
+                await self.set_token('connect')
+            else:
+                await self.set_token('vwg')
+
             response = await self.get(url, vin)
-            # Pre-heater, ???
+            # Pre-heater on older cars
             if response.get('requestStatusResponse', {}).get('status', False):
                 result = response.get('requestStatusResponse', {}).get('status', False)
-            # For electric charging, climatisation and departure timers
+            # Electric charging, climatisation and departure timers
             elif response.get('action', {}).get('actionState', False):
                 result = response.get('action', {}).get('actionState', False)
+            # Skoda native API requests
+            elif response.get('status', False):
+                result = response.get('status', False)
             else:
                 result = 'Unknown'
             # Translate status messages to meaningful info
-            if result == 'request_in_progress' or result == 'queued' or result == 'fetched':
+            if result in ['request_in_progress', 'queued', 'fetched', 'InProgress', 'Waiting']:
                 status = 'In progress'
-            elif result == 'request_fail' or result == 'failed':
+            elif result in ['request_fail', 'failed']:
                 status = 'Failed'
-            elif result == 'unfetched':
+            elif result in ['unfetched', 'delayed', 'PollingTimeout']:
                 status = 'No response'
-            elif result == 'request_successful' or result == 'succeeded':
+            elif result in [ "FailPlugDisconnected", "FailTimerChargingActive" ]:
+                status = "Unavailable"
+            elif result in ['request_successful', 'succeeded', "Successful"]:
                 status = 'Success'
             else:
                 status = result
@@ -955,7 +1019,7 @@ class Connection:
             response = await self.get(self._make_url(urls.get(action), vin = vin))
             secToken = response['securityPinAuthInfo']['securityToken']
             challenge = response['securityPinAuthInfo']['securityPinTransmission']['challenge']
-            spinHash = self.hash_spin(challenge, spin)
+            spinHash = self.hash_spin(challenge, str(spin))
             body = {
                 'securityPinAuthentication': {
                     'securityPin': {
@@ -980,7 +1044,7 @@ class Connection:
  #### Data set functions ####
     async def dataCall(self, query, vin='', **data):
         """Function to execute actions through VW-Group API."""
-        if self.logged_in == False:
+        if not self.logged_in:
             if not await self.doLogin():
                 _LOGGER.warning(f'Login for {BRAND} account failed!')
                 raise SkodaLoginFailedException(f'Login for {BRAND} account failed')
@@ -1013,6 +1077,7 @@ class Connection:
             _LOGGER.error(f'Failure to execute: {error}')
         return False
 
+   # VW-Group API methods
     async def setRefresh(self, vin):
         """"Force vehicle data update."""
         try:
@@ -1037,7 +1102,7 @@ class Connection:
         """Start/Stop charger."""
         try:
             await self.set_token('vwg')
-            response = await self.dataCall(f'fs-car/bs/batterycharge/v1/{BRAND}/{COUNTRY}/vehicles/$vin/charger/actions', vin, json = data)
+            response = await self.dataCall(f'fs-car/bs/batterycharge/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/charger/actions', vin, json = data)
             if not response:
                 raise SkodaException('Invalid or no response')
             elif response == 429:
@@ -1052,14 +1117,15 @@ class Connection:
             raise
         return False
 
-    async def setTimers(self, vin, data, spin):
+    async def setDeparturetimer(self, vin, data, spin):
         """Set departure timers."""
+        _LOGGER.debug(f"Set timers with data {data}")
         try:
             # First get most recent departuretimer settings from server
-            departuretimers = await self.getTimers(vin)
-            timer = departuretimers.get('timers', {}).get('timersAndProfiles', {}).get('timerList', {}).get('timer', [])
-            profile = departuretimers.get('timers', {}).get('timersAndProfiles', {}).get('timerProfileList', {}).get('timerProfile', [])
-            setting = departuretimers.get('timers', {}).get('timersAndProfiles', {}).get('timerBasicSetting', [])
+            departuretimers = await self.getDeparturetimer(vin)
+            timer = departuretimers.get('departuretimers', {}).get('timersAndProfiles', {}).get('timerList', {}).get('timer', [])
+            profile = departuretimers.get('departuretimers', {}).get('timersAndProfiles', {}).get('timerProfileList', {}).get('timerProfile', [])
+            setting = departuretimers.get('departuretimers', {}).get('timersAndProfiles', {}).get('timerBasicSetting', [])
 
             # Construct Timer data
             timers = [{},{},{}]
@@ -1082,7 +1148,7 @@ class Connection:
                 timerid = int(data.get('id'))-1 if data.get('id', False) else 0
                 # Set timer programmed status if data contains action = on or off
                 if data.get('action', None) in ['on', 'off']:
-                    action = 'programmed' if data.get('action', False) else 'notProgrammed'
+                    action = 'programmed' if data.get('action', False) == 'on' else 'notProgrammed'
                 # Set departure schedule
                 elif data.get('action', None) == 'schedule':
                     action = 'programmed' if data.get('schedule', {}).get('enabled', False) else 'notProgrammed'
@@ -1248,6 +1314,37 @@ class Connection:
             raise
         return False
 
+   # Skoda native API request methods
+    async def setSkodaAPI(self, endpoint, vin, data):
+        try:
+            url = f"https://api.connect.skoda-auto.cz/api/v1/{endpoint}/operation-requests?vin={vin}"
+            response = await self.dataCall(url, vin, json = data)
+            if not response:
+                raise SkodaException('Invalid or no response')
+            else:
+                request_id = response.get('id', 0)
+                request_state = response.get('status', 'unknown')
+                _LOGGER.debug(f'Request returned with state "{request_state}", request id: {request_id}')
+                return dict({'id': str(request_id), 'state': request_state})
+        except:
+            raise
+        return False
+
+    async def setAirConditioning(self, vin, data):
+        """Execute air conditioning actions."""
+        await self.set_token('connect')
+        return await self.setSkodaAPI('air-conditioning', vin, data)
+
+    async def setTimers(self, vin, data):
+        """Execute timer actions."""
+        await self.set_token('connect')
+        return await self.setSkodaAPI('air-conditioning', vin, data)
+
+    async def setCharging(self, vin, data):
+        """Execute charging actions."""
+        await self.set_token('connect')
+        return await self.setSkodaAPI('charging', vin, data)
+
  #### Token handling ####
     @property
     async def validate_tokens(self):
