@@ -408,11 +408,11 @@ class ElectricClimatisation(Switch):
         return self.vehicle.electric_climatisation
 
     async def turn_on(self):
-        await self.vehicle.set_climatisation('electric')
+        await self.vehicle.set_climatisation(mode = 'electric')
         await self.vehicle.update()
 
     async def turn_off(self):
-        await self.vehicle.set_climatisation('off')
+        await self.vehicle.set_climatisation(mode = 'off')
         await self.vehicle.update()
 
     @property
@@ -436,11 +436,11 @@ class AuxiliaryClimatisation(Switch):
         return self.vehicle.auxiliary_climatisation
 
     async def turn_on(self):
-        await self.vehicle.set_climatisation('auxiliary', self.spin)
+        await self.vehicle.set_climatisation(mode = 'auxiliary', spin = self.spin)
         await self.vehicle.update()
 
     async def turn_off(self):
-        await self.vehicle.set_climatisation('off')
+        await self.vehicle.set_climatisation(mode = 'off')
         await self.vehicle.update()
 
     @property
@@ -502,6 +502,33 @@ class WindowHeater(Switch):
         return dict(last_result = self.vehicle.climater_action_status)
 
 
+class SeatHeating(Switch):
+    def __init__(self):
+        super().__init__(attr="seat_heating", name="Seat Heating", icon="mdi:seat-recline-normal")
+
+    @property
+    def state(self):
+        return self.vehicle.seat_heating
+
+    async def turn_on(self):
+        #await self.vehicle.set_seat_heating('start')
+        #await self.vehicle.update()
+        pass
+
+    async def turn_off(self):
+        #await self.vehicle.set_seat_heating('stop')
+        #await self.vehicle.update()
+        pass
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        return dict(last_result = self.vehicle.climater_action_status)
+
+
 class BatteryClimatisation(Switch):
     def __init__(self):
         super().__init__(attr="climatisation_without_external_power", name="Climatisation from battery", icon="mdi:power-plug")
@@ -706,6 +733,7 @@ def create_instruments():
         TrunkLock(),
         RequestUpdate(),
         WindowHeater(),
+        SeatHeating(),
         BatteryClimatisation(),
         ElectricClimatisation(),
         AuxiliaryClimatisation(),
@@ -784,6 +812,18 @@ def create_instruments():
             icon="mdi:battery-charging-100",
             unit="h",
         ),
+        Sensor(
+            attr="charging_power",
+            name="Charging power",
+            icon="mdi:flash",
+            unit="W"
+        ),
+        Sensor(
+            attr="charge_rate",
+            name="Charging rate",
+            icon="mdi:battery-heart",
+            unit="km/h"
+        ),
         Sensor(
             attr="electric_range",
             name="Electric range",
@@ -814,6 +854,12 @@ def create_instruments():
             icon="mdi:thermometer",
             unit="°C",
         ),
+        Sensor(
+            attr="climatisation_time_left",
+            name="Climatisation time left",
+            icon="mdi:clock",
+            unit="h",
+        ),
         Sensor(
             attr="trip_last_average_speed",
             name="Last trip average speed",
@@ -1029,7 +1075,6 @@ def create_instruments():
 
 class Dashboard:
     def __init__(self, vehicle, **config):
-        _LOGGER.debug("Setting up dashboard with config :%s", config)
         self.instruments = [
             instrument
             for instrument in create_instruments()
